/* Debug Header */
#if DEBUG
# define IFDEBUG(x) do { if (debug) x; }while(0)
# define IFDEBUG5(x) do { if (debug) x; }while(0)
#else
# define IFDEBUG(x)
# define IFDEBUG5(x)
#endif
/* End Debug Header */

#define TIMEOUT 10
#define CONTIMEOUT 0
#define CONTIME 500000

#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netdb.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "extern.h"

#define NET_BUFFER 8192

/* net_connect(): 
 * accepts							returns
 * hostname (char *)
 * port number (int)
 *											socket file descriptor number
 *
 * this function is just to open a socket
 * to a remote host given the hostname and
 * port number.  It returns the fd number of
 * the open socket.
 */
int net_connect(const char *hostname, const int port, int *err) {

	int sockfd, connection, ret;
	int error=0;
	struct hostent *ipaddr;
	struct sockaddr_in servaddr;
	struct timeval tv; 
	fd_set fdw, fdr;
	int i;

	*err=0;
	IFDEBUG(print_debug(4, "net_connect(): hostname=%s, port=%i\n", hostname, port));
	ipaddr=gethostbyname(hostname);
	if (!ipaddr) {
		for(i=0;i<5;i++) {
			ipaddr=gethostbyname(hostname);
			if (ipaddr) break;
			sleep(1);
		}
		herror("Fatal Error");
		exit(1);
	}
	sockfd=socket(PF_INET, SOCK_STREAM, 0);
	fcntl(sockfd, F_SETFL, O_NONBLOCK);

	servaddr.sin_family=ipaddr->h_addrtype;
	servaddr.sin_port=htons(port);
	memcpy((char *) &servaddr.sin_addr.s_addr, ipaddr->h_addr_list[0], ipaddr->h_length);
	connection=connect(sockfd, (struct sockaddr *)&servaddr, sizeof(servaddr));

	tv.tv_sec = CONTIMEOUT; 
	tv.tv_usec = CONTIME; 

	if (connection) {
		*err=-1;
		switch (errno) {
			case EAGAIN: /* we can handle this error */
			case EINPROGRESS: 

				FD_ZERO(&fdw);
				FD_SET(sockfd, &fdw);
				fdr = fdw ;
	
				tv.tv_sec = CONTIMEOUT; 
				tv.tv_usec = CONTIME; 

				if (select(sockfd + 1, &fdr, &fdw, NULL, &tv ) != 0) {
					/* even if socket is writeable, may be an error, need explicit check */
					*err=0;
					if (FD_ISSET(sockfd, &fdr) || FD_ISSET(sockfd, &fdw)) {
						int len = sizeof(error);
						*err=1;
						if (getsockopt(sockfd, SOL_SOCKET, SO_ERROR, (char *)&error, &len) == 0) {
							/* solaris getsockopt() will return -1 if an
 								 * error has occurred  otherwise we must
 								 * explicitly check "error"
							 */
							if (error == 0) { /* no errors on connect */

								/* set tiemouts on read/writes */
								tv.tv_sec = TIMEOUT; 
								tv.tv_usec = 0; 

								setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)); 
								setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv)); 
														
								break; /* let's get out of here ... */
							}
						}
					}
				}
				/* Fall through to default */
			default: 
				close(sockfd); /* close socket, and (maybe) try again */
				sockfd=0;
				break;
		}
	}
	return sockfd;
}

char *net_read(int sockfd) {	

	int num, r;
	char *str;

	fd_set rdfs;
	struct timeval tv;
	int ret, timeout, i;
	char *buf, *ptr;
	
	unsigned char t[1];


	/* clear string contents and initialise all to null. Good and safe practise */	
	str=calloc(sizeof(char), NET_BUFFER+1);	

	FD_ZERO(&rdfs);
	FD_SET(sockfd, &rdfs);

	timeout = TIMEOUT;
	tv.tv_sec = timeout;
	tv.tv_usec = 0;

	ret=select((sockfd+1), &rdfs, NULL, NULL, &tv);
	// IFDEBUG5(print_debug(5, "Select() returned: %i\n", ret));
	if (ret) {
		num=read(sockfd, str, NET_BUFFER);

		if (num<0) {
			printf("net_read(): An Error Occurred\n");
			perror("An Error Occurred");
			num=0;
		}
	}

	if (!num || !ret)	{
		free(str);
		return NULL; /* safely returned something worth while. */	
	}


	IFDEBUG5( { \
		printf("\nDebug[5]: net_read(%i) ", num); \
		for(r=0;r<num;r++) { \
			t[0]=*(str+r); \
			printf("%hhi ", t[0]); \
		} \
		printf("\n"); \
	} )

	/* return the damn stuff.  */	
	return str;
}

int net_send(int sockfd, char *str) {
	
	int r;

	IFDEBUG(print_debug(4, "Sending: %s", str));
	r=send(sockfd, str, strlen(str), 0);
	if (!r) {
		printf("net_send(): An Error Occurred\n");
		perror("net_send()");
	}
	return(r);
}

int net_nsend(int sockfd, char *str, int len) {
	
	int r;
	unsigned char t[1];

	IFDEBUG5( { \
		printf("Debug[5]: net_nsend() "); \
		for(r=0;r<len;r++) { \
			t[0]=*(str+r); \
			printf("%hhi ", t[0]); \
		} \
		printf("\n"); \
	} )
	r=send(sockfd, str, len, 0);
	if (!send(sockfd, str, len, 0)) {
		printf("net_nsend(): An Error Occurred\n");
		perror("net_nsend()");
	}
	return(r);
}

char *getip(const char *hostname) {

	int sockfd, connection;
	struct hostent *ipaddr;
	struct sockaddr_in servaddr;
	int i;

	ipaddr=gethostbyname(hostname);
	if (!ipaddr) {
		for(i=0;i<5;i++) {
			ipaddr=gethostbyname(hostname);
			if (ipaddr) break;
			sleep(1);
		}
		if (!ipaddr) {
			fprintf(stderr, "Fatal Error unable to reverse map hostname %s", hostname);
			herror("in DNS");
			return((char *)hostname);
		}
	}
	return ipaddr->h_addr_list[0];
}

